#!/bin/bash
#

usage()
{
   echo "USAGE: `basename ${0}` <product_dir> <e1[0457]|c2> [<prof>] [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

set_local_qualifiers()
{
  [[ "${basequal}" =~ : ]] && \
   ssi_die "${package} build does not accept multiple qualifiers."
  cqual="${basequal}"
  extraqual=prof
}

set_compiler_flags()
{
  case ${cqual} in
    e1[0245])
      xcc=gcc
      local cc=gcc
      local cxx=g++
      local fc=gfortran
      local std=14
      ;;
    e1[79])
      xcc=gcc
      local cc=gcc
      local cxx=g++
      local fc=gfortran
      local std=17
      ;;
    e13)
      xcc=clang
      local cc=clang
      local cxx=clang++
      local fc=gfortran
      local std=14
      ;;
    c[25])
      xcc=clang
      local cc=clang
      local cxx=clang++
      local fc=gfortran
      local std=17
      extra_opts+=(-stdlib=libc++)
      ;;
    *)
      echo "ERROR: unrecognized compiler qualifier ${cqual}"
      usage
      exit 1
  esac
  compiler_specs=(-DCMAKE_C_COMPILER=${cc} \
      -DCMAKE_CXX_COMPILER=${cxx} \
      -DCMAKE_Fortran_COMPILER=${fc} \
      -DCMAKE_CXX_STANDARD=${std} \
      -DCMAKE_Fortran_FLAGS="-ffree-line-length-none"
      )
}

# -------------------------------------------------------------------
# Specific for this package
function make_osx_prodct_stub()
{
    mkdir -p ${pkgdir}/lib64 || ssi_die "Unable to make ${pkgdir}/lib64 for links."
    cd ${pkgdir}/lib64 || ssi_die "Unable to cd to ${pkgdir}/lib64 to make links."
#create softlinks here
    blas_path=/System/Library/Frameworks/Accelerate.framework/Versions/Current/Frameworks/vecLib.framework/Versions/A/libBLAS.dylib
    lapack_path=/System/Library/Frameworks/Accelerate.framework/Versions/Current/Frameworks/vecLib.framework/Versions/A/libLAPACK.dylib

    ln -sv ${blas_path} libblas.dylib || ssi_die "Unable to link BLAS library from ${blas_path}"
    ln -sv ${lapack_path} liblapack.dylib || ssi_die "Unable to link LAPACK library from ${lapack_path}"

    ${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual} || ssi_die "Unable to declare product."

    setup_and_verify

    # this must be last
    if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib64 ]
    then
       ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
    fi
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
maketar=${3}

if [[ "${maketar:-tar}" != "tar" ]]; then
  echo "WARNING: build type specifier ${maketar} no longer accepted: ignoring." 1>&2
  maketar=${4}
fi

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=lapack
origpkgver=v3_8_0
pkgver=${origpkgver}b
ssibuildshims_version=v1_04_08
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' -e 's/^v//'`
pkgtarfile=lapack-${pkgdotver}.tar.gz

extra_opts=()

get_this_dir
get_ssibuildshims
set_local_qualifiers
source define_basics NO_DEBUG_SOURCE

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib64 ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

# make a product stub for OSX
if [ "${OS1}" = "Darwin" ]
then
    make_osx_prodct_stub
    exit 0
fi

# Nothing below is run on OSX...

buildtype=MinSizeRel

mkdir -p ${pkgdir} || ssi_die "Failed to create ${pkgdir}"
mkdir -p ${blddir} || ssi_die "Failed to create ${blddir}"
mkdir -p ${sourcedir} || ssi_die "Failed to create ${sourcedir}"

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || ssi_die "fake setup failed"

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`
source ${SSIBUILDSHIMS_DIR}/bin/setup_cmake "${product_dir}" || ssi_die "failed to setup cmake"

make=make
makejarg=(-j ${ncore})

ups active

cd ${sourcedir} || ssi_die "Unable to cd to ${sourcedir}"
tar xf ${tardir}/${pkgtarfile} || ssi_die "Unable to unwind ${tardir}/${pkgtarfile}"

cd ${sourcedir}/lapack-${pkgdotver} || ssi_die "Unable to cd to ${sourcedir}/lapack-${pkgdotver}"

set_compiler_flags

set -x

cd ${blddir} || ssi_die "Unable to cd to ${blddir}"

# We would wish to use -std=2008ts, but LAPACK source code doesn't
# support it. Fortunately it doesn't tickle anything that would make it
# binary incompatible with anything we would do downstream.
if [ ${xcc} = "gcc" ]
then
echo "Building with gcc: $(which gcc) and g++: $(which g++)" 1>&2
elif [ ${xcc} = "clang" ]
then
echo "Building with clang: $(which clang) and clang++: $(which clang++)" 1>&2
fi

common_opts=(-DCMAKE_BUILD_TYPE=${buildtype} \
    -DCMAKE_INSTALL_PREFIX=${LAPACK_FQ_DIR} \
    "${compiler_specs[@]}" \
    -DBUILD_SHARED_LIBS=ON \
    -DCBLAS=ON  \
    ${extra_opts:+"${extra_opts[@]}"}
)

cmake "${common_opts[@]}" \
 ${sourcedir}/lapack-${pkgdotver}

(( $? == 0 )) || ssi_die "cmake step failed"

${make} "${makejarg[@]}" || ssi_die "${make} failed."
ctest -j $ncore || ssi_die "Failed in ctest"
make -j $ncore install || ssi_die "Failed to install"

# The Ubuntu build likes to use 'lib' as the subdirectory of
# $LAPACK_FQ_DIR, while SLF likes to use lib64. We'll use lib64.
if [ -f ${LAPACK_FQ_DIR}/lib/liblapack.so ]
then
  mkdir -p ${LAPACK_FQ_DIR}/lib64
  mv ${LAPACK_FQ_DIR}/lib/* ${LAPACK_FQ_DIR}/lib64/
  rm -r ${LAPACK_FQ_DIR}/lib
fi

set +x

[[ -f "${LAPACK_LIB}/liblapack.so.${pkgdotver}" ]] || ssi_die "failed to populate ${LAPACK_LIB} \n" \
  "       Something is very wrong"

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual} || \
  ssi_die "failed to declare ${package} ${pkgver} -q ${fullqual}"

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------


# fix paths for pkgconfig
pkgcnf=`ls ${LAPACK_FQ_DIR}/lib64/pkgconfig/*.pc`
for file in ${pkgcnf}
do
  mv ${file} ${file}.bak
  cat ${file}.bak | sed -e "s%${LAPACK_FQ_DIR}%\${LAPACK_FQ_DIR}%g" > ${file}
done

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS} || ssi_die "setup failed"
echo "$package is installed at $(eval echo "\${${package^^*}_FQ_DIR}")"

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${LAPACK_LIB} ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
